import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_story/ocg_story.dart';
import 'package:ocg_story/src/models/basic.dart';
import 'package:ocg_story/src/widgets/story_create_widget.dart';
import 'package:ocg_story/src/widgets/text_add_widget.dart';

class StoryImageCreateScreen extends StatefulWidget {
  final StoryImageData imageData;
  final Function(StoryImage image) onCreate;
  final bool allowAdvanced;

  const StoryImageCreateScreen(
      {Key key,
      @required this.imageData,
      this.onCreate,
      this.allowAdvanced = false})
      : super(key: key);
  @override
  State<StatefulWidget> createState() => StoryImageCreateScreenState();
}

class StoryImageCreateScreenState extends State<StoryImageCreateScreen> {
  List<StoryBasic> _children = [];
  final GlobalKey<ScaffoldState> _key = GlobalKey<ScaffoldState>();
  List<Widget> _stackChildren = [];
  StoryCreateController _controller;
  bool _isCreating;
  bool _isDragging;

  @override
  void initState() {
    super.initState();
    _children.add(widget.imageData);
    _isCreating = false;
    _isDragging = false;
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _offset = Offset(MediaQuery.of(context).size.width / 2 - 32.0,
          MediaQuery.of(context).size.height - 150);
      setState(() {});
    });
  }

  Offset _offset;
  bool _isAdding = false;
  @override
  Widget build(BuildContext context) {
    if (_offset == null) {
      return Container();
    }
    debugPrint("Stack");

    return AnnotatedRegion<SystemUiOverlayStyle>(
      value: SystemUiOverlayStyle.light,
      child: ModalProgressHUD(
        inAsyncCall: _isCreating,
        child: Scaffold(
          key: _key,
          backgroundColor: Colors.transparent,
          body: SafeArea(
            child: Stack(
              fit: StackFit.expand,
              children: [
                Visibility(
                  visible: !_isAdding,
                  maintainState: true,
                  child: StoryCreateWidget(
                    isAdvanced: widget.allowAdvanced,
                    children: _children,
                    deleteOffset: _offset,
                    onBasicTap: _onBasicTap,
                    onDragged: (isDragging) {
                      debugPrint("Dragging");
                      setState(() {
                        _isDragging = isDragging;
                      });
                    },
                    onCompleted: (_controller) {
                      this._controller = _controller;
                    },
                  ),
                ),
                Visibility(
                  visible: !_isAdding,
                  child: _buildOptionsScreen(),
                  maintainState: true,
                ),
                Visibility(
                  visible: true,
                  child: Stack(
                    children: _stackChildren,
                  ),
                )
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildButton(IconData iconData, Function onPressed) {
    return Padding(
      padding: const EdgeInsets.all(6.0),
      child: IconButton(
        onPressed: onPressed,
        color: Colors.black26,
        icon: Icon(
          iconData,
          color: Colors.white,
        ),
      ),
    );
  }

  Widget _buildOptionsScreen() {
    debugPrint("Print");
    return Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
        child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: <Widget>[
              _isDragging
                  ? Container()
                  : Container(
                      padding: EdgeInsets.all(4),
                      decoration: BoxDecoration(
                          color: Colors.black12,
                          boxShadow: [
                            BoxShadow(
                                color: Colors.black12,
                                blurRadius: 12,
                                spreadRadius: 1)
                          ]),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          IconButton(
                              icon: Icon(
                                Icons.clear,
                                color: Colors.white,
                              ),
                              onPressed: () {
                                Navigator.of(context).maybePop();
                              }),
                          Row(
                            mainAxisSize: MainAxisSize.min,
                            children: <Widget>[
                              widget.allowAdvanced
                                  ? _buildButton(Icons.text_fields, onTextAdd)
                                  : Container(
                                      width: 0,
                                      height: 0,
                                    ),
                              // widget.allowAdvanced
                              //     ? _buildButton(Icons.tag_faces, () {
                              //         setState(() {
                              //           _adding = true;
                              //           _stackChildren.removeLast();
                              //           _stackChildren.add(SmileyAddWidget(
                              //             onSelected: _onSelected,
                              //           ));
                              //         });
                              //       })
                              //     : Container(
                              //         width: 0,
                              //         height: 0,
                              //       ),
                              widget.onCreate != null
                                  ? _buildButton(Icons.check, () async {
                                      setState(() {
                                        _isCreating = true;
                                      });
                                      AppResponse<StoryImageData>
                                          imageResponse = await createStory();
                                      _isCreating = false;
                                      if (mounted) {
                                        setState(() {});
                                        if (imageResponse.isSuccess) {
                                          StoryImageData imageData =
                                              imageResponse.data;
                                          StoryImage storyImage =
                                              StoryImage.named(
                                                  url: imageData.imagePath);
                                          widget.onCreate(storyImage);
                                        } else {
                                          _key.currentState.showSnackBar(
                                              SnackBar(
                                                  content: Text(
                                                      "Some error occurred")));
                                        }
                                      }
                                    })
                                  : Container(
                                      width: 0,
                                      height: 0,
                                    ),
                            ],
                          )
                        ],
                      ),
                    ),
              Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Center(
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.baseline,
                      textBaseline: TextBaseline.alphabetic,
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        Container(),
                      ],
                    ),
                  ))
            ]));
  }

  void onTextAdd({StoryTextData textData}) {
    setState(() {
      _isAdding = true;
      if (_stackChildren.length > 0) _stackChildren.removeLast();
      _stackChildren.add(TextAddWidget(
        onSelected: _onSelected,
        data: textData,
      ));
    });
  }

  Future<AppResponse<StoryImageData>> createStory() async {
    if (_controller != null) {
      AppResponse<File> response = await _controller.createStory();
      if (response.isSuccess) {
        return AppResponse.named(
            data: StoryImageData.named(imagePath: response.data.path));
      } else {
        return AppResponse.named(error: "${response.error}");
      }
    } else {
      return AppResponse.named(error: "Controller not null");
    }
  }

  void _onBasicTap(StoryBasic basic) {
    if (basic is StoryTextData) {
      setState(() {});
      onTextAdd(textData: basic);
    }
  }

  void _onSelected(StoryBasic basic) {
    setState(() {
      _isAdding = false;
      if (basic != null) _children.add(basic);
      _stackChildren.removeLast();
    });
  }
}
