import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

class MovingWidget extends StatefulWidget {
  final Widget child;
  final bool canRemove;
  final int id;
  final Function() onRemove;
  final Function(bool isDragging) onDragged;
  const MovingWidget(
      {Key key,
      @required this.child,
      @required this.onRemove,
      this.canRemove,
      this.id = -1,
      this.onDragged})
      : super(key: key);
  @override
  State<StatefulWidget> createState() => _MovingWidgetState();
}

class _MovingWidgetState extends State<MovingWidget> {
  Offset globalPosition;
  final GlobalKey _key = GlobalKey();
  double opacity = 1.0;
  Size _size = Size(0, 0);
  bool _isDragging = false;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      RenderBox _renderBox = _key.currentContext?.findRenderObject();
      if (_renderBox != null) {
        _size = _renderBox.size;
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    if (globalPosition == null) {
      globalPosition = Offset(MediaQuery.of(context).size.width / 2,
          MediaQuery.of(context).size.height / 2);
    }
    return Positioned(
      key: _key,
      top: globalPosition.dy,
      left: globalPosition.dx,
      child: Draggable<int>(
        data: widget.id,
        onDragEnd: (details) {
          Offset offset = details.offset - Offset(0, _size.height / 2);
          _onChange(offset);
        },
        onDragStarted: () {
          _onDragging(true);
        },
        onDragCompleted: () {
          _onDragging(false);
        },
        onDraggableCanceled: (Velocity velocity, Offset offset) {
          _onDragging(false);
        },
        feedback: Opacity(
            opacity: opacity,
            child: Material(
              child: widget.child,
              color: Colors.transparent,
            )),
        child: Opacity(opacity: opacity, child: widget.child),
        childWhenDragging: Container(),
      ),
    );
  }

  void _onChange(Offset position) {
    setState(() {
      globalPosition = position;
    });
    _onDragging(false);
  }

  void _onDragging(bool isDragging) {
    if (isDragging && !_isDragging) {
      _isDragging = true;
      if (widget.onDragged != null) {
        widget.onDragged(_isDragging);
      }
    } else if (!isDragging && _isDragging) {
      _isDragging = false;
      if (widget.onDragged != null) {
        widget.onDragged(_isDragging);
      }
    }
  }

  bool get isRemovable => widget.canRemove;
}
