import 'dart:io';
import 'dart:ui';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:ocg_story/src/widgets/story_widget.dart';

class StoryImageWidget extends StatefulWidget {
  final String image;
  final Function() onLoaded;
  final StoryAssetType type;
  final Duration defaultDuration;
  const StoryImageWidget(
      {Key key,
      @required this.image,
      @required this.onLoaded,
      this.type = StoryAssetType.Network,
      this.defaultDuration})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _StoryImageWidgetState();
}

class _StoryImageWidgetState extends State<StoryImageWidget> {
  @override
  void initState() {
    super.initState();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (widget.type != StoryAssetType.Network) widget.onLoaded();
  }

  Duration get duration => widget.defaultDuration ?? Duration(seconds: 2);

  @override
  Widget build(BuildContext context) {
    Widget image;
    if (widget.type == StoryAssetType.File) {
      image = Image.file(
        File(widget.image),
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
      );
    } else if (widget.type == StoryAssetType.Asset) {
      image = Image.asset(
        widget.image,
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
      );
    } else {
      image = CachedNetworkImage(
        imageUrl: widget.image,
        imageBuilder: (context, imageProvider) {
          return LoadedImageWidget(
            onLoaded: widget.onLoaded,
            provider: imageProvider,
            url: widget.image,
          );
        },
        errorWidget: (context, url, error) => Center(
          child: Icon(Icons.error),
        ),
        filterQuality: FilterQuality.high,
        progressIndicatorBuilder: (context, child, downloadProgress) {
          if (downloadProgress == null) {
            return Container();
          }
          double progress = downloadProgress?.progress;
          if (progress == null) {
            return Container();
          }
          return Center(
            child: CircularProgressIndicator(
              value: progress,
            ),
          );
        },
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
      );
    }
    return Container(
      width: MediaQuery.of(context).size.width,
      height: MediaQuery.of(context).size.height,
      child: image,
    );
  }
}

class LoadedImageWidget extends StatefulWidget {
  final ImageProvider provider;
  final Function() onLoaded;
  final String url;
  LoadedImageWidget({Key key, this.provider, this.onLoaded, this.url})
      : super(key: key);

  @override
  _LoadedImageWidgetState createState() => _LoadedImageWidgetState();
}

class _LoadedImageWidgetState extends State<LoadedImageWidget> {
  @override
  void initState() {
    super.initState();
    widget.onLoaded();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  void didUpdateWidget(LoadedImageWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return Image(image: widget.provider);
  }
}
